clc;                             % clear command window/console
clear;                           % clear workspace                                      
close all;                       % close all windows (plots)

import casadi.*

% create empty optimization problem
opti = casadi.Opti();

% define variables
x = opti.variable();                    % scalar by default. This is the same as opti.variable(1,1).
y = opti.variable(1,1);                 % explicitly define as 1x1 matrix, i.e. scalar

% define objective
f = x^2 - 2*x + y + y^2;

% hand objective to casadi, no minimization done yet
opti.minimize(f)                        % opti.minimize(x^2 - 2*x + y + y^2) would also work

% define constraints. To include several constraints, just call this
% function several times
opti.subject_to(   x >= 1.5);
opti.subject_to( x+y >= 0  );

% define solver
opti.solver('ipopt')                    % Use IPOPT as solver
% opti.solver('sqpmethod')              % Use sqpmethod as solver

% solve optimization problem

sol = opti.solve();

% read and print solution
xopt = sol.value(x);
yopt = sol.value(y);

disp(' ')
if strcmp(opti.stats.return_status, 'Solve_Succeeded')      % if casadi returns sucessfull
    disp(['Optimal solution found: x = ' num2str(xopt) ', y = ' num2str(yopt)]);
else
    disp('Problem failed')
end

%% plot solution
% Define the function
[x, y] = meshgrid(-5:0.1:5, -5:0.1:5);
f = x.^2 - 2*x + y + y.^2;

% Create a contour plot for f
contour(x, y, f, 20); % You can adjust the number of contour lines as needed

% Add the line x + y = 0 to the plot
hold on;
x_line = -5:0.1:5;
y_line = -x_line;
plot(x_line, y_line, 'k', 'LineWidth', 1.5); 
xline(1.5, 'k', 'LineWidth', 1.5);
plot(xopt,yopt,'r.','MarkerSize',25)
% LaTeX-style labels and title
xlabel('$x$', 'Interpreter', 'latex');
ylabel('$y$', 'Interpreter', 'latex');
% title('$f(x, y) = x^2 - 2x + y + y^2$ and $x + y = 0$', 'Interpreter', 'latex');

axis equal
% Hold off to stop overlaying on the same plot
hold off;
